/*
  This program goes with the ATTiny kit used in University of Wyoming ECE workshops.
  The circuit is built in two stages.  
  1. A photo resistor is attached to analog in. The program is designed to turn on one
     of two different LEDs depending on light level.
  2. The photoresistor is removed and replaced by an LED. This system is designed to run one
     of two games: simon says or whack a mole.
   In both cases, the power up condition is to run a Blink program.
   The program checks to see if photoresistor present, if so it runs light program (1). If not
   it runs in game mode.
   If in game mode, a pushbutton determines whether to run simon says or whack a mole
   Connections:
    GoLed: is LED to ground
    leds[k]: is LED to ground
    photoresPin: resistor to ground and photoresistor to +5   

   Whack-a-mole Rules:  computer flashes an LED briefly. Player must push proper button while light is on, otherwise you lose.
     Losing is indicated by rapid flashing
     Successful whacking is solid led..
   Simon says rules:
     Computer flashes a light sequence.  When center GoLed is on, push buttons in proper order.

   R. Kubichek
   ECE Department, University of Wyoming
*/

//#define UNO  //comment this out to run on ATTiny

#ifdef UNO
int buttons[] = {9, 8}; //pin numbers for pushbuttons
int leds[] = {A1, 2}; //LED pins
int GoLed = 3;  //LED lights to request input
#else  //ATTINY assignments
int buttons[] = {3, 4}; //pin numbers for pushbuttons
int leds[] = {A1, 0}; //LED pins
int GoLed = 1;  //LED lights to request input
#endif
int photoresPin = A1; //on ATTINY A1 same as D2 (pin 7)

int Game0;
int Game1;

int nlight = 0; //# values to remember
const int MaxLight = 10;  //# to remember to win
int Lights[MaxLight];
const long Maxt = 10000; //max time ms allowed to key in the sequence

//Declare your variables here
int mole1LED = leds[0];      //LED for first mole
int mole2LED = leds[1];      //LED for second mole
int mole1Switch = buttons[0];  //mole 1 switch
int mole2Switch = buttons[1];   //mole 2 switch
int successLED = GoLed; //LED if successful mole whack
int failLED = GoLed;    //LED if miss mole


void setup() {
  randomSeed(millis());     //initialize random number generator
  pinMode(buttons[0], INPUT_PULLUP);
  pinMode(buttons[1], INPUT_PULLUP);
  pinMode(leds[0], OUTPUT);
  pinMode(leds[1], OUTPUT);
  pinMode(GoLed, OUTPUT);
  Game0 = digitalRead(buttons[0]);
  Game1 = digitalRead(buttons[1]);
  while (Game0 && Game1) { //hang in BLINK until buttonpush
    Game0 = digitalRead(buttons[0]);
    Game1 = digitalRead(buttons[1]);
    blinkk(GoLed, 1, 1000);  //demo the blink program
  }

  if (checkpin(photoresPin) == 2) { //photoresistor is attached so do this program
    //program #2:  photoresistor demo
    //assumes connected +V to photoresPin to 220 to gnd.
    blinkk(GoLed, 10, 50); //signify a program change
    pinMode(photoresPin, INPUT);
    int thresh = analogRead(photoresPin) - 4; //get ambient light
    //delay(1000); blinkk(GoLed,thresh,100); delay(1000);
    while (1) { //hang here
      int val = analogRead(photoresPin);
  //delay(500); blinkk(GoLed,val,100); delay(500);
      if (val < thresh) blinkk(leds[1], 1, 100); //arbitrary threshold
      else blinkk(GoLed, 1, 100);
    }
  }
}

void loop() {
  if (Game0) {  //whack a mole
    //Begins whack a mole
    while (1) { //whacka mole
      if (random(2) == 0) { //Flip Coin: random returns value 0 or 1.
        moleWhack(mole1Switch, mole2Switch, mole1LED);  //Mole is at position 1
      }
      else {              //mole is at position 2
        moleWhack(mole2Switch, mole1Switch, mole2LED);
      }
      delay(random(1000, 4000));  //wait a random time before a new mole
    }
  }
  else {
    //Begins Simon Says
    while (1) {
      addLight();
      digitalWrite(GoLed, HIGH); //request user input
      checkUserInput();
      digitalWrite(GoLed, LOW); //request user input
    }
  }
}
void moleWhack(int moleSwitch,int wrongMole,  int moleLED) {
  //mole is at position given by moleSwitch and moleLED
  //check if mole gets whacked
  digitalWrite(moleLED, HIGH);  //turn on the mole LED
  int moleWait = random(200, 700); //random amount of time for whack to occur
  long startTime = millis();      //save the start time
  while (millis() - startTime < moleWait) { //stay in loop until timer expires
    if (digitalRead(moleSwitch) == LOW && digitalRead(wrongMole)==HIGH) { //pushed=whacked!
      digitalWrite(successLED, HIGH);  //blink on
      digitalWrite(moleLED, LOW);      //turn off mole LED. He's been whacked
      delay(500);
      digitalWrite(successLED, LOW);   //blink off
      return;  //this terminates the function
    }
  }
  //Timed out - If we get here then mole was not whacked
  delay(100);
  digitalWrite(moleLED, LOW);           //turn off mole light to prepare for next mole
  blinkk(failLED, 5, 50); //fail fail fail
  blinkk(mole1LED, 5, 50);
  blinkk(mole2LED, 5, 50);
}


void blinkk(int pin, int nblink, int del) {
  for (int k = 0; k < nblink; k++) {
    digitalWrite(pin, HIGH);
    delay(del);
    digitalWrite(pin, LOW);
    delay(del / 2);
  }
}

int checkpin(int pinno) { 
  //used to determine how breadboard set up: photodetector mode or game mode
  //These values determined empirically
  //returns 1 if pin is open (shouldn't happen)
  //returns 2 if pin connected to photoresistor voltage divider
  //returns 3 if pin connected to LED+resistor
  pinMode(pinno, INPUT_PULLUP); //assumes pinno is an analog pin
  int x = analogRead(pinno);
  if (x < 200){
    pinMode(pinno,INPUT);  //set to be an input, light detector mode
    return 2; //photoresistor
  }
  if (x < 900) {
   pinMode(pinno,OUTPUT); //get set to be an output, game mode
   digitalWrite(pinno,LOW); //turn off pullup
   return 3; //LED
  }
  return 1; //x will be close to 1000 since pullup resistor engaged
}

void addLight() {
  if (nlight + 1 > MaxLight) {
    blinkk(leds[0],10,30);
    blinkk(leds[1],10,30); //you win!!
    blinkk(leds[0],10,30);
    blinkk(leds[1],10,30); //you win
    EndGame();
    return;
  }
  Lights[nlight] = millis()%2;  //random result is 0 or 1
  nlight = nlight + 1;
  ShowSequence();  //show player the sequence
}

void checkUserInput() {
  int answer;
  long t = millis(); //start the clock
  for (int k = 0; k < nlight; k++) {
    int correct = Lights[k];  //correct button
    while (digitalRead(buttons[0]) && digitalRead(buttons[1])) { //hang here until button push
      if (millis() - t > Maxt) { //time exceeded
        blinkk(leds[correct], 15, 50); //show correct response
        EndGame();
        return; //too slow
      }
    }
    delay(30); //switch bounce delay
    if (digitalRead(buttons[1]) == LOW) {
      answer = 1;
    }
    else {
      answer = 0;
    }
    while(!digitalRead(buttons[0]) || !digitalRead(buttons[1])) {}; //wait for button release before proceeding
    
    if (answer != correct) { //wrong answer
      blinkk(leds[correct], 15, 50); //show correct response
      EndGame();
      return;
    }
    else blinkk(leds[correct],1,50);  //correct. blink briefly and continue
  }  //done with this round
}
void ShowSequence() {
  delay(1000);
  for (int k = 0; k < nlight; k++) {
    blinkk(leds[Lights[k]], 1, 300);
  }
  delay(1000);
}

void EndGame() {
  digitalWrite(GoLed, LOW); //request user input
  digitalWrite(leds[0], HIGH);
  digitalWrite(leds[1], HIGH);
  delay(1000);
  digitalWrite(leds[0], LOW);
  digitalWrite(leds[1], LOW);
  nlight = 0; //set up for a new game
  delay(5000);
}
